@@ -1,11 +1,19 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
 import copy
+from functools import lru_cache
 
 import numpy as np
 
 import astropy.units as u
-from astropy.coordinates import ITRS, CartesianRepresentation, SphericalRepresentation
+from astropy.coordinates import (
+    ITRS,
+    BaseBodycentricRepresentation,
+    BaseCoordinateFrame,
+    BaseGeodeticRepresentation,
+    CartesianRepresentation,
+    SphericalRepresentation,
+)
 from astropy.utils import unbroadcast
 
 from .wcs import WCS, WCSSUB_LATITUDE, WCSSUB_LONGITUDE
@@ -13,23 +21,55 @@ from .wcs import WCS, WCSSUB_LATITUDE, WCSSUB_LONGITUDE
 __doctest_skip__ = ["wcs_to_celestial_frame", "celestial_frame_to_wcs"]
 
 __all__ = [
-    "obsgeo_to_frame",
     "add_stokes_axis_to_wcs",
     "celestial_frame_to_wcs",
-    "wcs_to_celestial_frame",
-    "proj_plane_pixel_scales",
-    "proj_plane_pixel_area",
-    "is_proj_plane_distorted",
-    "non_celestial_pixel_scales",
-    "skycoord_to_pixel",
-    "pixel_to_skycoord",
-    "custom_wcs_to_frame_mappings",
     "custom_frame_to_wcs_mappings",
-    "pixel_to_pixel",
-    "local_partial_pixel_derivatives",
+    "custom_wcs_to_frame_mappings",
     "fit_wcs_from_points",
+    "is_proj_plane_distorted",
+    "local_partial_pixel_derivatives",
+    "non_celestial_pixel_scales",
+    "obsgeo_to_frame",
+    "pixel_to_pixel",
+    "pixel_to_skycoord",
+    "proj_plane_pixel_area",
+    "proj_plane_pixel_scales",
+    "skycoord_to_pixel",
+    "wcs_to_celestial_frame",
 ]
 
+SOLAR_SYSTEM_OBJ_DICT = {
+    "EA": "Earth",
+    "SE": "Moon",
+    "ME": "Mercury",
+    "VE": "Venus",
+    "MA": "Mars",
+    "JU": "Jupiter",
+    "SA": "Saturn",
+    "UR": "Uranus",
+    "NE": "Neptune",
+}
+
+
+@lru_cache(maxsize=100)
+def solar_system_body_frame(object_name, representation_type):
+    return type(
+        f"{object_name}Frame",
+        (BaseCoordinateFrame,),
+        dict(name=object_name, representation_type=representation_type),
+    )
+
+
+@lru_cache(maxsize=100)
+def solar_system_body_representation_type(
+    object_name, baserepresentation, equatorial_radius, flattening
+):
+    return type(
+        f"{object_name}{baserepresentation.__name__[4:]}",
+        (baserepresentation,),
+        dict(_equatorial_radius=equatorial_radius, _flattening=flattening),
+    )
+
 
 def add_stokes_axis_to_wcs(wcs, add_before_ind):
     """
@@ -118,6 +158,42 @@ def _wcs_to_celestial_frame_builtin(wcs):
                 representation_type=SphericalRepresentation,
                 obstime=wcs.wcs.dateobs or None,
             )
+        elif xcoord[2:4] in ("LN", "LT") and xcoord[:2] in SOLAR_SYSTEM_OBJ_DICT.keys():
+            # Coordinates on a planetary body, as defined in
+            # https://agupubs.onlinelibrary.wiley.com/doi/10.1029/2018EA000388
+
+            object_name = SOLAR_SYSTEM_OBJ_DICT.get(xcoord[:2])
+
+            a_radius = wcs.wcs.aux.a_radius
+            b_radius = wcs.wcs.aux.b_radius
+            c_radius = wcs.wcs.aux.c_radius
+            if "bodycentric" in wcs.wcs.name.lower():
+                baserepresentation = BaseBodycentricRepresentation
+                representation_type_name = "BodycentricRepresentation"
+            else:
+                baserepresentation = BaseGeodeticRepresentation
+                representation_type_name = "GeodeticRepresentation"
+
+            if a_radius == b_radius:
+                equatorial_radius = a_radius * u.m
+                flattening = (a_radius - c_radius) / a_radius
+            else:
+                raise NotImplementedError(
+                    "triaxial systems are not supported at this time."
+                )
+
+            # create a new representation class
+            representation_type = solar_system_body_representation_type(
+                SOLAR_SYSTEM_OBJ_DICT.get(xcoord[:2]),
+                baserepresentation,
+                equatorial_radius,
+                flattening,
+            )
+
+            # create a new frame class
+            frame = solar_system_body_frame(
+                SOLAR_SYSTEM_OBJ_DICT.get(xcoord[:2]), representation_type
+            )
         else:
             frame = None
 
@@ -163,6 +239,25 @@ def _celestial_frame_to_wcs_builtin(frame, projection="TAN"):
         ycoord = "TLAT"
         wcs.wcs.radesys = "ITRS"
         wcs.wcs.dateobs = frame.obstime.utc.isot
+    # TODO: once we have a BaseBodyFrame, replace this with an isinstance check
+    elif hasattr(frame, "name") and frame.name in SOLAR_SYSTEM_OBJ_DICT.values():
+        xcoord = frame.name[:2].upper().replace("MO", "SE") + "LN"
+        ycoord = frame.name[:2].upper().replace("MO", "SE") + "LT"
+        if issubclass(frame.representation_type, BaseGeodeticRepresentation):
+            wcs.wcs.name = "Planetographic Body-Fixed"
+        elif issubclass(frame.representation_type, BaseBodycentricRepresentation):
+            wcs.wcs.name = "Bodycentric Body-Fixed"
+        else:
+            raise ValueError(
+                "Planetary coordinates in WCS require a geodetic or bodycentric "
+                "representation, not {frame.representation_type}."
+            )
+        wcs.wcs.aux.a_radius = frame.representation_type._equatorial_radius.value
+        wcs.wcs.aux.b_radius = frame.representation_type._equatorial_radius.value
+        wcs.wcs.aux.c_radius = wcs.wcs.aux.a_radius * (
+            1.0
+            - frame.representation_type._flattening.to(u.dimensionless_unscaled).value
+        )
     else:
         return None
 
@@ -177,7 +272,7 @@ FRAME_WCS_MAPPINGS = [[_celestial_frame_to_wcs_builtin]]
 
 class custom_wcs_to_frame_mappings:
     def __init__(self, mappings=[]):
-        if hasattr(mappings, "__call__"):
+        if callable(mappings):
             mappings = [mappings]
         WCS_FRAME_MAPPINGS.append(mappings)
 
@@ -194,7 +289,7 @@ custom_frame_mappings = custom_wcs_to_frame_mappings
 
 class custom_frame_to_wcs_mappings:
     def __init__(self, mappings=[]):
-        if hasattr(mappings, "__call__"):
+        if callable(mappings):
             mappings = [mappings]
         FRAME_WCS_MAPPINGS.append(mappings)
 
@@ -881,7 +976,7 @@ def local_partial_pixel_derivatives(wcs, *pixel, normalize_by_world=False):
         derivatives[:, i] = world_off - world_ref
 
     if normalize_by_world:
-        derivatives /= derivatives.sum(axis=0)[:, np.newaxis]
+        derivatives /= derivatives.sum(axis=1)[:, np.newaxis]
 
     return derivatives
 
@@ -1003,7 +1098,8 @@ def fit_wcs_from_points(
     Parameters
     ----------
     xy : (`numpy.ndarray`, `numpy.ndarray`) tuple
-        x & y pixel coordinates.
+        x & y pixel coordinates.  These should be in FITS convention, starting
+        from (1,1) as the center of the bottom-left pixel.
     world_coords : `~astropy.coordinates.SkyCoord`
         Skycoord object with world coordinates.
     proj_point : 'center' or ~astropy.coordinates.SkyCoord`
@@ -1044,8 +1140,8 @@ def fit_wcs_from_points(
     if (type(proj_point) != type(world_coords)) and (proj_point != "center"):
         raise ValueError(
             "proj_point must be set to 'center', or an"
-            + "`~astropy.coordinates.SkyCoord` object with "
-            + "a pair of points."
+            "`~astropy.coordinates.SkyCoord` object with "
+            "a pair of points."
         )
 
     use_center_as_proj_point = str(proj_point) == "center"
@@ -1085,15 +1181,14 @@ def fit_wcs_from_points(
     if type(projection) == str:
         if projection not in proj_codes:
             raise ValueError(
-                "Must specify valid projection code from list of "
-                + "supported types: ",
+                "Must specify valid projection code from list of supported types: ",
                 ", ".join(proj_codes),
             )
         # empty wcs to fill in with fit values
         wcs = celestial_frame_to_wcs(frame=world_coords.frame, projection=projection)
     else:  # if projection is not string, should be wcs object. use as template.
         wcs = copy.deepcopy(projection)
-        wcs.cdelt = (1.0, 1.0)  # make sure cdelt is 1
+        wcs.wcs.cdelt = (1.0, 1.0)  # make sure cdelt is 1
         wcs.sip = None
 
     # Change PC to CD, since cdelt will be set to 1
@@ -1101,7 +1196,7 @@ def fit_wcs_from_points(
         wcs.wcs.cd = wcs.wcs.pc
         wcs.wcs.__delattr__("pc")
 
-    if (type(sip_degree) != type(None)) and (type(sip_degree) != int):
+    if (sip_degree is not None) and (type(sip_degree) != int):
         raise ValueError("sip_degree must be None, or integer.")
 
     # compute bounding box for sources in image coordinates:
